#ifndef FrameCPP_VERSION_8_FrHistory_HH
#define FrameCPP_VERSION_8_FrHistory_HH

#include "framecpp/FrameCPP.hh"

#if ! defined(SWIGIMPORTED)
#include "framecpp/Version7/FrHistory.hh"
#endif /* ! defined(SWIGIMPORTED) */

#if ! defined(SWIGIMPORTED)
#include "framecpp/Version8/FrameSpec.hh"
#include "framecpp/Version8/STRING.hh"
#endif /* ! defined(SWIGIMPORTED) */

namespace FrameCPP
{
  namespace Version_8
  {
    //===================================================================
    /// \brief History Structure Definition
    //===================================================================
    class FrHistory
      : public ObjectWithChecksum< Common::FrameSpec::Object >
    {
    public:
      typedef INT_4U time_t;

      //-----------------------------------------------------------------
      /// \brief Default constructor
      //-----------------------------------------------------------------
      FrHistory( );

      //-----------------------------------------------------------------
      /// \brief Copy Constructor
      ///
      /// \param[in] Source
      ///     The object from which to copy the information.
      ///
      /// \return
      ///    A new instance of this object.
      //-----------------------------------------------------------------
      FrHistory( const FrHistory& Source );

      //-----------------------------------------------------------------
      /// \brief Constructor
      ///
      /// \param[in] Name
      ///     Name of the history record
      /// \param[in] Time
      ///     Time of post-processing
      ///     (GPS seconds since GPS standard epoch)
      /// \param[in] Comment
      ///     Program name and relevant comments needed to define
      ///     post-processing
      ///
      /// \return
      ///    A new instance of this object.
      //-----------------------------------------------------------------
      FrHistory( const std::string& Name,
		 INT_4U Time,
		 const std::string& Comment );

      //-----------------------------------------------------------------
      /// \brief Promotion Constructor
      ///
      /// \param[in] Source
      ///     An FrHistory structure from a previous frame specification.
      /// \param[in] Stream
      ///     The stream from which the earliest version of the
      ///     FrAdcData structure was read.
      ///
      /// \return
      ///     A new instance of an FrHistory object
      //-----------------------------------------------------------------
      FrHistory( Previous::FrHistory& Source, Common::IStream* Stream );

      //-----------------------------------------------------------------
      /// \brief The name structure name of this object.
      ///
      /// \return
      ///     The name of the structure as specified by the frame
      ///     specification.
      //-----------------------------------------------------------------
      static const char* const StructName( );

      //-----------------------------------------------------------------
      /// \brief The description of structure
      ///
      /// \return
      ///     A Description object which describes this structure as
      ///     specified by the frame specification.
      //-----------------------------------------------------------------
      static const Common::Description* StructDescription( );

      //-----------------------------------------------------------------
      /// \brief The name structure name of this object.
      ///
      /// \return
      ///     The name of the structure as specified by the frame
      ///     specification.
      //-----------------------------------------------------------------
      virtual const char* const ObjectStructName( ) const;

      virtual std::string ErrorInfo( ) const;

      //-----------------------------------------------------------------
      /// \brief Return the name associate with the FrHistory structure.
      ///
      /// \return
      ///     The name associated with the FrHistory structure
      //-----------------------------------------------------------------
      const std::string& GetName() const;

      //-----------------------------------------------------------------
      /// \brief Return the time of the post-processing.
      ///
      /// \return
      ///     The time of the post-processing.
      //-----------------------------------------------------------------
      INT_4U GetTime( ) const
      {
         return m_data.time;
      }   

      //-----------------------------------------------------------------
      /// \brief Return the description of the post-porcessing.
      ///
      /// \return
      ///     The description of the post-processing
      //-----------------------------------------------------------------
      const std::string& GetComment( ) const
      {
 	 return m_data.comment;
      }   

      //-----------------------------------------------------------------
      /// \brief Merge with another FrHistory object
      ///
      /// \param[in] RHS
      ///     The source of the information to append to this FrHistory
      ///     structure.
      ///
      /// \return
      ///     A reference to this object
      //-----------------------------------------------------------------
      FrHistory& Merge( const FrHistory& RHS );

      //-----------------------------------------------------------------
      /// \brief Promote previous version of object to this version
      ///
      /// \param[in] Source
      ///     The frame specification version of the object.
      /// \param[in] Obj
      ///     The previous version of the object.
      /// \param[in] Stream
      ///     Stream from which the origional object was created.
      ///
      /// \return
      ///     A new instance of FrameH based on Obj
      //-----------------------------------------------------------------
      static promote_ret_type
      Promote( INT_2U Source,
	       promote_arg_type Obj,
	       Common::IStream* Stream );

      //-----------------------------------------------------------------
      /// \brief equality operator
      ///
      /// \param[in] RHS
      ///     The FrHistory object to be compared.
      ///
      /// \return
      ///     The value true is returned if this object is equivelent
      ///     to the RHS instance; false otherwise.
      //-----------------------------------------------------------------
      bool operator==( const FrHistory& RHS ) const;

      //-----------------------------------------------------------------
      /// \brief equality operator for abstract data type
      ///
      /// \param[in] RHS
      ///     The object to be compared.
      ///
      /// \return
      ///     The value true is returned if this object is equivelent
      ///     to the RHS instance; false otherwise.
      //-----------------------------------------------------------------
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      //-----------------------------------------------------------------
      /// \brief Down grade an object
      ///
      /// \param[in] Target
      ///     The version of the frame specification to demote too.
      /// \param[in] Stream
      ///     The input stream from which the original object was read.
      ///
      /// \return
      ///     An object of the previous generation.
      //-----------------------------------------------------------------
      virtual demote_ret_type
      demote( INT_2U Target,
	      demote_arg_type Obj,
	      Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Upgrade an object
      ///
      /// \param[in] Target
      ///     The version of the frame specification to promote too.
      /// \param[in] Stream
      ///     The input stream from which the original object was read.
      ///
      /// \return
      ///     An object of this class.
      //-----------------------------------------------------------------
      virtual promote_ret_type
      promote( INT_2U Source,
	       promote_arg_type Obj,
	       Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Number of bytes needed to write this structure
      ///
      /// \param[in] Stream
      ///     The stream from which to the object is being read or
      ///     written.
      ///
      /// \return
      ///     The number of bytes need to read or write this object.
      //-----------------------------------------------------------------
      cmn_streamsize_type pBytes( const Common::StreamBase& Stream) const;

      //-----------------------------------------------------------------
      /// \brief Virtual constructor
      ///
      /// \param[in] Stream
      ///     The input stream from where the object is being read.
      //-----------------------------------------------------------------
      virtual FrHistory* pCreate( Common::IStream& Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Write the structure to the stream
      ///
      /// \param[in] Stream
      ///     The output stream where the object is to be written.
      //-----------------------------------------------------------------
      virtual void pWrite( Common::OStream& Stream ) const;

    private:
      //-----------------------------------------------------------------
      //-----------------------------------------------------------------
      typedef ObjectWithChecksum< Common::FrameSpec::Object > object_type;

      using Common::FrameSpec::Object::Create;

      struct fr_history_data_type {
	//---------------------------------------------------------------
	/// Name of the history record.
        /// \note
	///     When an FrHistory is linked to an FrProcData, its name
	///     variable must be the FrProcData channel name.
	//---------------------------------------------------------------
	STRING	name;
	//---------------------------------------------------------------
	/// Time of post-processing,
	/// GPS time in integer seconds since GPS standard epoch.
	//---------------------------------------------------------------
	time_t	time;
	//---------------------------------------------------------------
	/// Program name and relevant comments needed to define
	/// post-processing.
	//---------------------------------------------------------------
	STRING	comment;

	//---------------------------------------------------------------
	/// \brief equality operator
	///
	/// \param[in] RHS
	///     The fr_history_data_type object to be compared.
	///
	/// \return
	///     The value true is returned if this object is equivelent
	///     to the RHS instance; false otherwise.
	//---------------------------------------------------------------
	bool operator==( const fr_history_data_type& RHS ) const;
      };

      fr_history_data_type	m_data;

      //-----------------------------------------------------------------
      /// \brief Stream Constructor
      ///
      /// \param[in] Stream
      ///     The stream from which the object is being read.
      ///
      /// \return
      ///    A new instance of this object.
      //-----------------------------------------------------------------
      FrHistory( Common::IStream& Stream );
    };

    inline const char* const FrHistory::
    StructName( )
    {
      static const char* const name = "FrHistory";
      return name;
    }

    inline bool FrHistory::
    operator==( const FrHistory& RHS ) const
    {
      return ( m_data == RHS.m_data );
    }
  } // namespace - Version_8
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_8_FrHistory_HH */
